
class CustomStdout:
    def __init__(self,appRootDir,cmd,recode=0):
        import sys
        self.console = sys.stdout
        self.error = sys.stderr
        sys.stdout = self
        sys.stderr = self
        #sys.stderr = self
        self.appRootDir = appRootDir+'logs'
        # import datetime,os
        # self.procFlag = "%s_%s_%d" % (cmd,datetime.datetime.strftime(datetime.datetime.now(), '%H%M%S'),os.getpid())
        self.procFlag = cmd
        self.recode = recode
        self.msgText = []

    def write(self,outStr):
        try:
            if len(self.msgText) == 0:
                import datetime
                nowTimeStr = datetime.datetime.strftime(datetime.datetime.now(), '%Y-%m-%d %H:%M:%S')
                import os
                prevStr = "【%s】（%d）" % (nowTimeStr, os.getpid())
                self.msgText.append(prevStr)
                # self.console.write(prevStr)

            self.msgText.append(outStr)
            # self.console.write(outStr)
        except:
            import traceback
            traceback.print_exc()
        finally:
            try:
                if outStr.endswith('\n') and len(self.msgText)>0:
                    lineMsg = ''.join(self.msgText)
                    self.msgText.clear()
                    if 'NO CONSOLE' not in lineMsg:
                        self.console.write(lineMsg)
                    else:
                        lineMsg = lineMsg.replace('NO CONSOLE','*')
                    self.writeFile(lineMsg)
            except:
                import traceback
                traceback.print_exc()

    def flush(self): pass

    def writeFile(self, outStr):
        if self.recode == 0: return
        try:
            import datetime, os
            dir_path = self.appRootDir + '/' + datetime.datetime.strftime(datetime.datetime.now(), '%Y%m%d') +'/'
            if not os.path.exists(dir_path): os.makedirs(dir_path)
            file_path = dir_path +  self.procFlag + '.log'
            with open(file_path, 'a', encoding='utf-8') as f:
                f.write(outStr)
        except:
            import traceback
            traceback.print_exc()

# 加载配置
def load_config(project_name,cmd):
    import os
    import sys
    import configparser

    appRootDir = None
    if getattr(sys, 'frozen', False):
        appRootDir = os.path.dirname(sys.executable)
    elif __file__:
        appRootDir = os.path.dirname(os.path.realpath(sys.argv[0]))
    if appRootDir is None:
        print('程序运行路径异常,退出程序')
        os._exit(-1)

    if project_name in appRootDir:
        index = appRootDir.find(project_name)
        appRootDir = appRootDir[0:index + len(project_name)]
    if appRootDir.endswith('/') is False:
        appRootDir = appRootDir + '/'
    appRootDir = appRootDir.replace('\\', '/')
    printer = CustomStdout(appRootDir, cmd)
    print("应用(%s)(%d) 根目录路径: %s" % (project_name,os.getpid(), appRootDir))

    configList = []
    for file in os.listdir(appRootDir):
        file_path = appRootDir + file
        if os.path.isfile(file_path) and os.path.splitext(file_path)[1] == '.ini':
            configList.append(file_path)


    config = configparser.RawConfigParser()
    config.read(filenames=configList, encoding='utf8')

    sections = config.sections()
    for section in sections:
        options = config.options(section)
        for option in options:
            var = config.get(section,option)
            if var.startswith('./'):
                var = appRootDir + var[2:]
                config.set(section,option,var)
            # print(section, option, var)

    printer.recode = config.getint('log','recode')

    return appRootDir, config





'''
数据库工具类
'''

import os
import traceback
import threading
import json
import datetime
import decimal

pid_index = '%02d' % (os.getpid() % 99)
mLock = threading.RLock()

# 获取当前时间戳 毫秒
def _getCurrentTimestamp():
    import time
    timestamp = int(round(time.time() * 1000))
    return timestamp

def _getCurrentTimestampSec():
    import time
    timestamp = int(time.time())
    return timestamp


########################################################### 每秒产生最多9999个ID
lastTime = _getCurrentTimestamp()
seqIndex = 1
# 时间戳生成唯一ID
def genTimestampDataID():
    global mLock,lastTime,seqIndex
    mLock.acquire()
    try:
        curTime = _getCurrentTimestamp()
        if curTime - lastTime < 1000 and seqIndex <= 9999:
            num_str = '%04d' % seqIndex
            seqIndex = seqIndex + 1
            return '%s%s%s' % (lastTime, num_str, pid_index)
        else:
            lastTime = curTime
            seqIndex = 1
            return genTimestampDataID()

    except Exception as e:
        traceback.print_exc()
        print('获取时间戳产生的数据标识错误',e)
    finally:
        mLock.release()

############################################################## 每秒产生最多999个ID且可设置后缀
lastTimeSec = _getCurrentTimestampSec()
seqIndexSec = 1
# 时间戳生成唯一ID
def genTimestampDataIDSec(suffix='0000%s'% pid_index ):
        global mLock,lastTimeSec,seqIndexSec
        mLock.acquire()
        try:
            curTimeSec = _getCurrentTimestampSec()
            if curTimeSec == lastTimeSec and seqIndexSec <= 999:
                num_str = '%03d' % seqIndexSec
                seqIndexSec = seqIndexSec + 1
                return '%s%s%s' % (lastTimeSec, num_str, suffix)
            else:
                while True:
                    if _getCurrentTimestampSec() > curTimeSec: break
                lastTimeSec = _getCurrentTimestampSec()
                seqIndexSec = 1
                return genTimestampDataIDSec(suffix)
        except Exception as e:
            traceback.print_exc()
            print('获取时间戳产生的数据标识错误', e)
        finally:
            mLock.release()


# 数据行转map
def rowConvertDict(rows,*dictKeys):
    '''按照顺序对 指定键 赋值'''
    dict = {}
    if type(dictKeys[0]) == list:
        dictKeys = dictKeys[0]
    try:
        for i in range(len(dictKeys)):
            try:
                keyObj = dictKeys[i]
                if type(keyObj) == str:
                    _v = rows[i]
                    if isinstance(_v,datetime.datetime):
                        _v = datetime.datetime.strftime(_v, '%Y-%m-%d %H:%M:%S')
                    dict[keyObj] = str(_v if _v is not None else '')


                if type(keyObj) == tuple:
                    kstr = keyObj[0]
                    index = keyObj[1]
                    if len(keyObj) >= 3:
                        func = keyObj[2]
                    else:
                        func = lambda row_index_v: row_index_v
                    _v = func(rows[index])
                    dict[kstr] = str(_v if _v is not None else '')
            except :
                traceback.print_exc()
    except:
        traceback.print_exc()
    return dict

def rowConvertDict_str(rows,dictKeys_str):
    return rowConvertDict(rows,dictKeys_str.split(","))

def linesConvertDict(lines,descriptionList):
    list = []
    for rows in lines:
        maps = rowConvertDict(rows, descriptionList)
        list.append(maps)
    return list

def pauseCreateTableDDL(createDDL, isPrint=False):
    try:
        from ddlparse import DdlParse
        createDDL = createDDL.replace('Nullable(DateTime)', 'DateTime')
        createDDL = createDDL.replace('Array(String)', 'String')
        table = DdlParse().parse(createDDL, source_database=DdlParse.DATABASE.mysql)
        fieldTupleList = []
        for col in table.columns.values():
            if isPrint : print(table.name, col.name, col.data_type, col.comment)
            fieldTupleList.append((col.comment, col.name, col.data_type))
        return table.name , fieldTupleList
    except: return None,None




class _MysqlRowDateToJsonEncoder(json.JSONEncoder):
    def default(self, obj):
            if isinstance(obj, datetime.datetime):
                return obj.strftime('%Y-%m-%d %H:%M:%S')
            elif isinstance(obj, datetime.date):
                return obj.strftime("%Y-%m-%d")
            elif isinstance(obj,datetime.time):
                return obj.strftime("%H:%M:%S")
            elif isinstance(obj, datetime.timedelta):
                return (datetime.datetime.min + obj).time().strftime("%H:%M:%S")
            elif isinstance(obj, decimal.Decimal):
                return str(decimal.Decimal(obj).quantize(decimal.Decimal('0.00')))
            elif isinstance(obj, bytes):
                return str(obj, encoding='utf-8')
            elif isinstance(obj,list):
                return json.dumps(obj, cls=_MysqlRowDateToJsonEncoder,ensure_ascii=False)
            elif isinstance(obj,dict):
                return json.dumps(obj, cls=_MysqlRowDateToJsonEncoder,ensure_ascii=False)
            else:
                return json.JSONEncoder.default(self, obj)

def MysqlRowDateToJsonEncoderFunc(rowsDict):
    return json.dumps(rowsDict, cls=_MysqlRowDateToJsonEncoder,ensure_ascii=False)




_db_mng_map = {}


def MYSQL_SET__GROUP_CONCAT_MAX_LEN(mysql_db, maxLen):
    '''设置mysql group_concat 函数 最大字符数量'''
    try:
        lines = mysql_db.query("show variables like 'group_concat_max_len';")
        if len(lines) == 1:
            curLen = int(lines[0][0])
            if curLen>=maxLen: return
        mysql_db.execute("SET GLOBAL group_concat_max_len = %d;" % maxLen)
        mysql_db.execute("SET SESSION group_concat_max_len = %d;" % maxLen)
        print('mysql concat_group max len: %d ' % maxLen)
    except: pass

def setting_database(database_obj):
    if database_obj.databaseType == 'mysql':
        MYSQL_SET__GROUP_CONCAT_MAX_LEN(database_obj, 4294967295)


def load_database_config(config):
    from modules.lsptoolbox.database.class_define import DatabaseClearThread,DatabaseManagerBean
    db_clear_thread = DatabaseClearThread()
    # print('连接检测线程',db_clear_thread)
    for session in  config.sections():
        if session.startswith('dbload::'):
            mngName = session.replace('dbload::','',1)
            database_obj = DatabaseManagerBean(config.get(session, 'dbtype'),
                                                       config.get(session, 'host'),
                                                       config.get(session, 'port'),
                                                       config.get(session, 'username'),
                                                       config.get(session, 'password'),
                                                       config.get(session, 'database'),
                                                       db_clear_thread)
            setting_database(database_obj)
            _db_mng_map[mngName] = database_obj
            print("加载数据库管理 %s,%s" % (mngName, str(_db_mng_map[mngName])))


def getDatabaseOperation(databaseName):
    mng = _db_mng_map.get(databaseName,None)
    if mng == None: raise ValueError("找不到指定数据库对象:%s" % databaseName)
    return mng




import setuptools

# setup说明 https://packaging.python.org/en/latest/tutorials/packaging-projects/

# python -m build
# python  -m twine upload --repository pypi dist/*

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name = "lsptoolbox",
    version = "0.0.3",
    author = "lee bottle",
    author_email = "793065165@qq.com",
    description="my python project tool box.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/15608447849/lsp-toolbox",
    package_dir={"": "modules"},
    packages=setuptools.find_packages("modules"),

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
